@@ -36,7 +36,7 @@ else:  # pragma: no cover
     input_ = input
 
 
-__version__ = '0.3.1'
+__version__ = '0.3.2'
 __author__ = 'pahaz'
 
 
@@ -59,15 +59,13 @@ _DEPRECATIONS = {
 DEFAULT_LOGLEVEL = logging.ERROR  #: default level if no logger passed (ERROR)
 TRACE_LEVEL = 1
 logging.addLevelName(TRACE_LEVEL, 'TRACE')
+DEFAULT_SSH_DIRECTORY = '~/.ssh'
 
-if os.name == 'posix':
-    DEFAULT_SSH_DIRECTORY = '~/.ssh'
-    UnixStreamServer = socketserver.UnixStreamServer
-else:
-    DEFAULT_SSH_DIRECTORY = '~/ssh'
-    UnixStreamServer = socketserver.TCPServer
+_StreamServer = socketserver.UnixStreamServer if os.name == 'posix' \
+    else socketserver.TCPServer
 
 #: Path of optional ssh configuration file
+DEFAULT_SSH_DIRECTORY = '~/.ssh'
 SSH_CONFIG_FILE = os.path.join(DEFAULT_SSH_DIRECTORY, 'config')
 
 ########################
@@ -352,7 +350,7 @@ class _ForwardHandler(socketserver.BaseRequestHandler):
                 src_addr=src_address,
                 timeout=TUNNEL_TIMEOUT
             )
-        except Exception as e:
+        except Exception as e:  # pragma: no cover
             msg_tupe = 'ssh ' if isinstance(e, paramiko.SSHException) else ''
             exc_msg = 'open new channel {0}error: {1}'.format(msg_tupe, e)
             log_msg = '{0} {1}'.format(self.info, exc_msg)
@@ -437,15 +435,15 @@ class _ThreadingForwardServer(socketserver.ThreadingMixIn, _ForwardServer):
     daemon_threads = _DAEMON
 
 
-class _UnixStreamForwardServer(UnixStreamServer):
+class _StreamForwardServer(_StreamServer):
     """
-    Serve over UNIX domain sockets (does not work on Windows)
+    Serve over domain sockets (does not work on Windows)
     """
 
     def __init__(self, *args, **kwargs):
         self.logger = create_logger(kwargs.pop('logger', None))
         self.tunnel_ok = queue.Queue(1)
-        UnixStreamServer.__init__(self, *args, **kwargs)
+        _StreamServer.__init__(self, *args, **kwargs)
 
     @property
     def local_address(self):
@@ -472,8 +470,8 @@ class _UnixStreamForwardServer(UnixStreamServer):
         return self.RequestHandlerClass.remote_address[1]
 
 
-class _ThreadingUnixStreamForwardServer(socketserver.ThreadingMixIn,
-                                        _UnixStreamForwardServer):
+class _ThreadingStreamForwardServer(socketserver.ThreadingMixIn,
+                                    _StreamForwardServer):
     """
     Allow concurrent connections to each tunnel
     """
@@ -832,9 +830,9 @@ class SSHTunnelForwarder(object):
     def _make_ssh_forward_server_class(self, remote_address_):
         return _ThreadingForwardServer if self._threaded else _ForwardServer
 
-    def _make_unix_ssh_forward_server_class(self, remote_address_):
-        return _ThreadingUnixStreamForwardServer if \
-            self._threaded else _UnixStreamForwardServer
+    def _make_stream_ssh_forward_server_class(self, remote_address_):
+        return _ThreadingStreamForwardServer if self._threaded \
+            else _StreamForwardServer
 
     def _make_ssh_forward_server(self, remote_address, local_bind_address):
         """
@@ -842,10 +840,9 @@ class SSHTunnelForwarder(object):
         """
         _Handler = self._make_ssh_forward_handler_class(remote_address)
         try:
-            if isinstance(local_bind_address, string_types):
-                forward_maker_class = self._make_unix_ssh_forward_server_class
-            else:
-                forward_maker_class = self._make_ssh_forward_server_class
+            forward_maker_class = self._make_stream_ssh_forward_server_class \
+                if isinstance(local_bind_address, string_types) \
+                else self._make_ssh_forward_server_class
             _Server = forward_maker_class(remote_address)
             ssh_forward_server = _Server(
                 local_bind_address,
@@ -1085,16 +1082,19 @@ class SSHTunnelForwarder(object):
         keys = SSHTunnelForwarder.get_agent_keys(logger=logger) \
             if allow_agent else []
 
-        if host_pkey_directories is not None:
-            paramiko_key_types = {'rsa': paramiko.RSAKey,
-                                  'dsa': paramiko.DSSKey,
-                                  'ecdsa': paramiko.ECDSAKey,
-                                  'ed25519': paramiko.Ed25519Key}
-            for directory in host_pkey_directories or [DEFAULT_SSH_DIRECTORY]:
-                for keytype in paramiko_key_types.keys():
-                    ssh_pkey_expanded = os.path.expanduser(
-                        os.path.join(directory, 'id_{}'.format(keytype))
-                    )
+        if host_pkey_directories is None:
+            host_pkey_directories = [DEFAULT_SSH_DIRECTORY]
+
+        paramiko_key_types = {'rsa': paramiko.RSAKey,
+                              'dsa': paramiko.DSSKey,
+                              'ecdsa': paramiko.ECDSAKey,
+                              'ed25519': paramiko.Ed25519Key}
+        for directory in host_pkey_directories:
+            for keytype in paramiko_key_types.keys():
+                ssh_pkey_expanded = os.path.expanduser(
+                    os.path.join(directory, 'id_{}'.format(keytype))
+                )
+                try:
                     if os.path.isfile(ssh_pkey_expanded):
                         ssh_pkey = SSHTunnelForwarder.read_private_key_file(
                             pkey_file=ssh_pkey_expanded,
@@ -1103,11 +1103,12 @@ class SSHTunnelForwarder(object):
                         )
                         if ssh_pkey:
                             keys.append(ssh_pkey)
+                except OSError as exc:
+                    if logger:
+                        logger.warning('Private key file {0} check error: {1}'
+                                       .format(ssh_pkey_expanded, exc))
         if logger:
-            logger.info('{0} keys loaded from host directory'.format(
-                len(keys))
-            )
-
+            logger.info('{0} key(s) loaded'.format(len(keys)))
         return keys
 
     @staticmethod
@@ -1455,12 +1456,12 @@ class SSHTunnelForwarder(object):
             _srv.shutdown()
             _srv.server_close()
             # clean up the UNIX domain socket if we're using one
-            if isinstance(_srv, _UnixStreamForwardServer):
+            if isinstance(_srv, _StreamForwardServer):
                 try:
                     os.unlink(_srv.local_address)
                 except Exception as e:
                     self.logger.error('Unable to unlink socket {0}: {1}'
-                                      .format(self.local_address, repr(e)))
+                                      .format(_srv.local_address, repr(e)))
         self.is_alive = False
         if self.is_active:
             self.logger.info('Closing ssh transport')
@@ -1862,7 +1863,7 @@ def _parse_arguments(args=None):
     return vars(parser.parse_args(args))
 
 
-def _cli_main(args=None):
+def _cli_main(args=None, **extras):
     """ Pass input arguments to open_tunnel
 
         Mandatory: ssh_address, -R (remote bind address list)
@@ -1894,6 +1895,9 @@ def _cli_main(args=None):
               logging.DEBUG,
               TRACE_LEVEL]
     arguments.setdefault('debug_level', levels[verbosity])
+    # do this while supporting py27/py34 instead of merging dicts
+    for (extra, value) in extras.items():
+        arguments.setdefault(extra, value)
     with open_tunnel(**arguments) as tunnel:
         if tunnel.is_alive:
             input_('''
